package Game 
{
	import flash.geom.Point;
	import net.flashpunk.Entity;
	import net.flashpunk.FP;
	import net.flashpunk.graphics.Spritemap;
	import net.flashpunk.Sfx;
	/**
	 * ...
	 * @author Andy Brown
	 */
	public class Actor extends GameObject 
	{
		
		public var sprites:Spritemap;
		
		public var movement:Point = new Point(0, 0);
		
		public var speed:Number = 200;
		
		public var direction:uint = 0;
		
		public var state:uint = ActorState.IDLE;
		
		public var attackTimer:Number = 0;
		public var hitTimer:Number = 0;
		public var invincableTimer:Number = 0;
		
		public var attackDelay:Number = 0;
		
		public var health:Number = 10;
		
		public var pushed:Number = 0;
		public var pushDir:Point = new Point(0, 0);
		
		
		private static const hitEnemySfx:Sfx = new Sfx(Assets.HitEnemy);
		
		private static const hitHeroSfx:Sfx = new Sfx(Assets.HeroHurt);
		
		private static const killEnemySfx:Sfx = new Sfx(Assets.KillMonster);
		
		public function Actor() 
		{
			
		}
		
		public function kill():void
		{
			if (this != GV.hero) {
				killEnemySfx.play();
				GV.kills ++;
				Kong.kongregate.stats.submit("Kills",1);
			}
		}
		
		public function hit(e:Entity = null, damage:int = 5, force:int = 1):void
		{
			if (invincableTimer > 0) return;
			
			
			
			pushDir.x = x - e.x;
			pushDir.y = y - e.y;
			
			pushDir.normalize(force);
			
			
			GV.bottomEmitter.emitBlood(x, y, pushDir);
			
			health -= 1;
			if (this == GV.hero)  {
				GV.gui.updateHealth();
				
				hitHeroSfx.play();
			} else {
				hitEnemySfx.play();
			}
			
			state = ActorState.HIT;
			hitTimer = 0.3;
			invincableTimer = 0.6;
		}
		
		override public function update():void
		{
			if (GV.gamePaused) return;
			super.update();
			
			
			layer = -y;
			
			movement.normalize(speed);
			
			if(state != ActorState.HIT) {
			moveBy(movement.x * FP.elapsed, movement.y * FP.elapsed, "solid");
			}
			
			if (attackDelay > 0) attackDelay -= FP.elapsed;
			if (invincableTimer > 0) invincableTimer -= FP.elapsed;
			
			switch(state) {
				case ActorState.IDLE:
					if (movement.x > 0 && movement.y == 0) {
						sprites.play("walkRight");
						direction = GC.RIGHT;
					}
					if (movement.x < 0 && movement.y == 0) {
						sprites.play("walkLeft");
						direction = GC.LEFT;
					}
					if (movement.y > 0) {
						sprites.play("walkDown");
						direction = GC.DOWN;
					}
					if (movement.y < 0) {
						sprites.play("walkUp");
						direction = GC.UP;
					}
					
					if (movement.x == 0 && movement.y == 0) {
						if (direction == GC.DOWN) sprites.play("standDown");
						if (direction == GC.UP) sprites.play("standUp");
						if (direction == GC.RIGHT) sprites.play("standRight");
						if (direction == GC.LEFT) sprites.play("standLeft");
					}
					break;
				case ActorState.ATTACKING:
					if (direction == GC.DOWN) sprites.play("attackDown");
					if (direction == GC.UP) sprites.play("attackUp");
					if (direction == GC.RIGHT) sprites.play("attackRight");
					if (direction == GC.LEFT) sprites.play("attackLeft");
					attackTimer -= FP.elapsed;
					if (attackTimer <= 0) state = ActorState.IDLE;
					break;
				case ActorState.HIT:
					hitTimer -= FP.elapsed;
					sprites.color = 0xFF0000;
					sprites.tinting = 0.5 + 0.05 * FP.rand(4);
					sprites.tintMode = 1;
					
					moveBy(200 * pushDir.x * FP.elapsed, 200 * pushDir.y * FP.elapsed, "solid");
					
					if (hitTimer <= 0) {
						sprites.tinting = 0;
						state = ActorState.IDLE;
						
						
						if (health <= 0) {
							kill();
							GV.bottomEmitter.emitDeath(x, y, pushDir);
							if(this != GV.hero) FP.world.remove(this);
						}
					}
					break;
			}
			
			
		}
		
	}

}